#ifndef INPUT_CLIPBOARD_H
#define INPUT_CLIPBOARD_H TRUE

/*
**  clipboard.h
**
**  (C) Copyright 2001-2003 Rocklyte Systems.
**      All Rights Reserved.
*/

#ifndef MAIN_H
#include <pandora/main.h>
#endif

/*****************************************************************************
** Clipboard class.
*/

#define VER_CLIPBOARD 1.0

/*****************************************************************************
** Methods
*/

#define MT_AddClipboardFiles   -1
#define MT_AddClipboardObject  -2
#define MT_AddClipboardObjects -3
#define MT_GetClipboardFiles   -4
#define MT_AddClipboardText    -5
#define MT_PasteClipboardFiles -6
#define MT_DeleteClip          -7

struct mtAddClipboardFiles {
   LONG   Datatype;          /* The datatype of the files or NULL if each file should be analysed */
   STRING Files;             /* The locations of the files to add */
   LONG   Flags;             /* Optional flags */
};

struct mtAddClipboardObject {
   LONG     Datatype;
   OBJECTID ObjectID;       /* The object containing the data to add */
   LONG     Flags;          /* Optional flags */
};

struct mtAddClipboardObjects {
   LONG     Datatype;
   OBJECTID *Objects;     /* Array of shared object ID's to add to the clipboard */
   LONG     Flags;        /* Optional flags */
};

struct mtAddClipboardText {
   STRING String;       /* String to add */
};

struct mtDeleteClip {
   LONG Datatype;
};

struct mtGetClipboardFiles {
   LONG   Datatype;  /* The type(s) of data recognised for retrieval (CLIP_TEXT, CLIP_AUDIO, CLIP_FILE ...) */
   LONG   Index;     /* Index of the item to return if Datatype is NULL */
   STRING *Files;    /* Result: The location(s) of the requested clip data, terminated with a NULL entry */
   LONG   Flags;     /* Result Flags: If the CLF_DELETE flag is returned, the location must be deleted after use (cut operation) */
};

struct mtPasteClipboardFiles {
   STRING Destination;
   LONG   Target;
};

#define CLF_DELETE  0x00000001 /* Set if the clipboard data should be deleted or Free()'d after a paste operation */
#define CLF_EXTEND  0x00000002 /* Instead of replacing the existing clipboard data, add the new data to what already exists */

/*****************************************************************************
** Clipboard types
*/

enum {
   CT_DATA=0,
   CT_AUDIO,
   CT_IMAGE,
   CT_FILE,
   CT_OBJECT,
   CT_TEXT,
   CT_END      /* End */
};

#define CLIP_DATA    (1<<CT_DATA)   // 1
#define CLIP_AUDIO   (1<<CT_AUDIO)  // 2
#define CLIP_IMAGE   (1<<CT_IMAGE)  // 4
#define CLIP_FILE    (1<<CT_FILE)   // 8
#define CLIP_OBJECT  (1<<CT_OBJECT) // 16
#define CLIP_TEXT    (1<<CT_TEXT)   // 32

#endif /* INPUT_CLIPBOARD_H */
