#ifndef INPUT_JOYPORT_H
#define INPUT_JOYPORT_H TRUE

/*
**  joyport.h
**
**  (C) Copyright 1996-2001 Rocklyte Systems.
**      All Rights Reserved.
*/

#ifndef MAIN_H
#include <pandora/main.h>
#endif

/*****************************************************************************
** JoyPort class.
*/

#define VER_JOYPORT 1.0

struct Joyport {
   OBJECT_HEADER       /* (R-) Class header */
   LONG   Port;        /* (R-) Port number - see JPORT definitions */
   FLOAT  XChange;     /* (R-) Reflects the most recent change in horizontal movement */
   FLOAT  YChange;     /* (R-) Reflects the most recent change in vertical movement */
   FLOAT  ZChange;     /* (R-) Reflects the most recent change in depth-based movement */
   FLOAT  LimitNorth;  /* (RI) Northern limit */
   FLOAT  LimitSouth;  /* (RI) Southern limit */
   FLOAT  LimitEast;   /* (RI) Eastern limit */
   FLOAT  LimitWest;   /* (RI) Western limit */
};

#define JD_BUTTON1    0x00000001L  /* Standard Fire Button (1) - LMB */
#define JD_BUTTON2    0x00000002L  /* Standard Fire Button (2) - RMB */
#define JD_BUTTON3    0x00000004L  /* Standard Fire Button (3) - MMB */
#define JD_BUTTON4    0x00000008L  /* "Start"    */
#define JD_BUTTON5    0x00000010L  /* "Select"   */
#define JD_BUTTON6    0x00000020L  /* Rewind  L1 */
#define JD_BUTTON7    0x00000040L  /* Forward R1 */
#define JD_BUTTON8    0x00000080L  /* Rewind  L2 */
#define JD_BUTTON9    0x00000100L  /* Forward R2 */

#define JD_DBLCLICK 0x80000000L  /* Second button-press qualifies as a double-click */

#define JD_DOUBLECLICK JD_DBLCLICK

#define JD_LMB         JD_BUTTON1
#define JD_RMB         JD_BUTTON2
#define JD_MMB         JD_BUTTON3

#define JD_DBLCLICK_LMB (JD_DBLCLICK|JD_BUTTON1)
#define JD_DBLCLICK_RMB (JD_DBLCLICK|JD_BUTTON2)
#define JD_DBLCLICK_MMB (JD_DBLCLICK|JD_BUTTON3)

/*****************************************************************************
** Available joy ports.  Note that on some systems, mouse and joyports can
** be plugged into the same port, so MOUSE1 and JOY1 could point to the
** same hardware device.  This is especially true on consoles and gaming
** computers.
*/

#define JPORT_MOUSE1   1
#define JPORT_MOUSE2   2
#define JPORT_MOUSE3   3
#define JPORT_MOUSE4   4
#define JPORT_JOY1     5
#define JPORT_JOY2     6
#define JPORT_JOY3     7
#define JPORT_JOY4     8
#define JPORT_JOY5     9
#define JPORT_JOY6     10
#define JPORT_JOY7     11
#define JPORT_JOY8     12

#define JPORT_MOUSE JPORT_MOUSE1

#endif /* INPUT_JOYPORT_H */
