#ifndef SYSTEM_CLASS_H
#define SYSTEM_CLASS_H TRUE

/*
**  class.h
**
**  (C) Copyright 1996-2001 Rocklyte Systems.
**      All Rights Reserved.
**
*/

#ifndef MAIN_H
#include <pandora/main.h>
#endif

#ifndef SYSTEM_FIELDS_H
#include <pandora/system/fields.h>
#endif

#ifndef SYSTEM_ACTIONS_H
#include <pandora/system/actions.h>
#endif

#define VER_CLASS 1.0

/*****************************************************************************
** Class categories.
*/

#define CCF_COMMAND    0x00000001
#define CCF_DRAWABLE   0x00000002
#define CCF_EFFECT     0x00000004
#define CCF_FILESYSTEM 0x00000008
#define CCF_GRAPHICS   0x00000010
#define CCF_GUI        0x00000020
#define CCF_IO         0x00000040
#define CCF_SYSTEM     0x00000080
#define CCF_TOOL       0x00000100
#define CCF_AUDIO      0x00000200
#define CCF_DATA       0x00000400
#define CCF_MISC       0x00000800
#define CCF_NETWORK    0x00001000
#define CCF_MULTIMEDIA 0x00002000

/****************************************************************************/

#define CLF_SHAREDONLY      0x00000001   /* Forces all objects of this class to be shared */
#define CLF_PRIVATEONLY     0x00000002   /* Do not allow public-memory instances of the class (private-shared objects wil still be possible) */
#define CLF_PROMOTECHILDREN 0x00000004   /* Promote class support for any child object defined in the class definition structure */
#define CLF_PUBLICOBJECTS   0x00000008   /* Forces all class objects to be allocated in public memory */

#define CLF_SHAREDOBJECTS   CLF_SHAREDONLY

#define MAX_FIELDNAME 19

struct Field {
   LONG  FieldID;                /* Provides a fast way of finding fields, e.g. FID_WIDTH */
   LONG  Offset;
   BYTE  Name[MAX_FIELDNAME+1];  /* The english name for the field, e.g. "Width" */
   LONG  Flags;                  /* Special flags that describe the field */
   LONG  Arg;                    /* Special argument */
   ERROR (*GetValue)(APTR Object, APTR Result);
   ERROR (*SetValue)(APTR Object, APTR Value);
};

struct FieldArray {
   BYTE Name[MAX_FIELDNAME+1]; /* The name of the field, e.g. "Width" */
   LONG FieldID;
   LONG Flags;                 /* Special flags that describe the field */
   LONG Arg;                   /* Special argument */
   APTR GetField;              /* void GetField(*Object, APTR Result); */
   APTR SetField;              /* ERROR SetField(*Object, APTR Value); */
};

/* This structure is used to define flags and lookup tables for fields */

struct FieldDef {
   STRING Name;
   LONG Value;
};

#define END_FIELD { "", 0, 0, 0, 0 }

struct ActionEntry {
   ERROR (*PerformAction)(OBJECTPTR, APTR);
};

/*****************************************************************************
** Field Definition Flags
*/

/*** Sizes/Types ***/

#define FD_DOUBLE    0x80000000
#define FD_LONG      0x40000000
#define FD_VARIABLE  0x20000000
#define FD_FLOAT     0x10000000
#define FD_POINTER   0x08000000
#define FD_LARGE     0x04000000
#define FD_UNLISTED  0x02000000

#define FT_POINTER  FD_POINTER
#define FT_FLOAT    FD_FLOAT
#define FT_LONG     FD_LONG
#define FT_DOUBLE   FD_DOUBLE
#define FT_LARGE    FD_LARGE
#define FT_STRING   (FD_POINTER|FD_STRING)
#define FT_UNLISTED FD_UNLISTED
#define FT_VARIABLE FD_VARIABLE

/*** Clarifiers ***/

#undef FD_READ
#undef FD_WRITE

#define FD_OBJECT     0x00000001
#define FD_CHILD      0x00000002
#define FD_STRING     0x00000004
#define FD_PERCENTAGE 0x00000008
#define FD_OBJECTID   0x00000010
#define FD_HEX        0x00000020
#define FD_FLAGS      0x00000040
#define FD_LOOKUP     0x00000080
#define FD_READ       0x00000100
#define FD_WRITE      0x00000200
#define FD_INIT       0x00000400

/*** Quick field definitions ***/

#define FDF_WORD       FD_WORD     /* Field is word sized (16-bit) */
#define FDF_LONG       FD_LONG     /* Field is long sized (32-bit) */
#define FDF_FLOAT      FD_FLOAT    /* Field is floating point sized (32-bit) */
#define FDF_DOUBLE     FD_DOUBLE   /* Field is double floating point sized (64-bit) */
#define FDF_LARGE      FD_LARGE    /* Field is large sized (64-bit) */
#define FDF_POINTER    FD_POINTER  /* Field is an address pointer (typically 32-bit) */
#define FDF_VARIABLE   FD_VARIABLE

#define FDF_OBJECT     (FD_POINTER|FD_OBJECT)  /* Field refers to another object */
#define FDF_OBJECTID   (FD_LONG|FD_OBJECTID)   /* Field refers to another object by ID */
#define FDF_CHILD      (FD_POINTER|FD_CHILD)   /* Field refers to a child object */
#define FDF_STRING     (FD_POINTER|FD_STRING)  /* Field points to a string */
#define FDF_STRINGID   (FD_LONG|FD_STRING)     /* Field points to a string located by memory ID */
#define FDF_PERCENTAGE (FD_FLOAT|FD_PERCENTAGE)
#define FDF_FLAGS      FD_FLAGS                /* Field contains flags */
#define FDF_HEX        FD_HEX                  /* Field is in hexadecimal */
#define FDF_LOOKUP     FD_LOOKUP               /* Lookup names for values in this field */
#define FDF_READ       FD_READ                 /* Field is readable */
#define FDF_WRITE      FD_WRITE                /* Field is writeable */
#define FDF_INIT       FD_INIT                 /* Field can only be written prior to Init() */
#define FDF_R          (FD_READ)
#define FDF_W          (FD_WRITE)
#define FDF_RW         (FD_READ|FD_WRITE)
#define FDF_RI         (FD_READ|FD_INIT)
#define FDF_I          (FD_INIT)
#define FDF_READWRITE  FDF_RW
#define FDF_READINIT   FDF_RI
#define FDF_READSET    FDF_RS
#define FDF_LONGFLAGS  (FDF_LONG|FDF_FLAGS)
#define FDF_FIELDTYPES (FD_FLOAT|FD_LONG|FD_DOUBLE|FD_LARGE|FD_POINTER|FD_VARIABLE)

/*****************************************************************************
** Class object (private).
*/

struct SysClass {
   OBJECT_HEADER                    /* -- (--) Standard header */
   struct MethodArray *Methods;     /* (RI) */
   struct FieldArray  *FieldArray;  /* (RI) */
   STRING Name;                     /* (R-) Name of the class, e.g. "Picture" */
   STRING FileExtension;            /* (RI) File extension */
   STRING FileDescription;          /* (RI) File description */
   LONG   ClassVersion;             /* (RI) Version of the class*/
   LONG   ObjectSize;               /* (RI) Byte-size of the class when produced as an object */
   LONG   Flags;                    /* (RI) Special flags */
   LONG   SubClassID;               /* (RI) Sub-class ID */
   LONG   BaseClassID;              /* (RI) Base-class ID */
   LONG   OpenCount;                /* (R-) Number of objects allocated to this class */
   LONG   TotalMethods;             /* (R-) Total number of methods */
   LONG   TotalFields;              /* (R-) Total number of fields */
   LONG   Category;
   STRING Usage;

   PRIVATE_FIELDS
   struct SysClass    *Prev;        /* Previous class in the chain */
   struct SysClass    *Next;        /* Next class in the chain */
   struct SysClass    *Base;        /* Reference to the base class if this is a sub-class*/
   struct Field       *Fields;      /* Internal field structure */
   struct FieldArray  *SubFields;   /* Extra fields defined by the sub-class */
   struct ActionEntry ActionTable[AC_END]; /* */
};

#endif /* SYSTEM_CLASS_H */
